function [Gred,Q,Efitrel]=LPVDMD(X,U,Y,dt,Etol,Nqmax,Alg)
% function [Gred,Q,Epod,Efit]=LPVDMD(X,U,Y,dt,Etol,Nqmax,Alg)
%
% This function implements the LPVDMD method for constructing a reduced
% order, discrete-time LPV system. The reduced order system is constructed
% from snapshots of the nonlinear system with a forced response. The modes
% can either be selected in a single step (Alg=1) or iteratively (Alg=2).
%
% Inputs:
%  *X = Nx-by-Nt-by-Ngrid matrix of state snapshots. X(:,i,j) is the
%       snapshot of the state at the i^th time taken at the j^th gridpoint.
%  *U = Nu-by-Nt-by-Ngrid matrix of input snapshots.
%  *Y = Ny-by-Nt-by-Ngrid matrix of output snapshots.
%  *dt = Sample time in sec.  It is assumed that the snapshots are
%     recorded every dt seconds.
%  *Etol = Relative error tolerance for model fit. Modes are selected
%     to achieve a model fit relative error below this tolerance (subject
%     to the maximum number specified by Nqmax).
%  *Nqmax = Maximum number of modes if Alg=1. If Alg=2 then this is
%     the maximum number of modes added at each grid point.
%     [Optional. Default is Nqmax = 10.]
%  *Alg = Flag used to select single-step mode selection (Alg=1) or
%     iterative selection (Alg=2).
%     [Optional. Default is Alg = 1.]
%
% Outputs:
%  *Gred = Ngrid-by-1 array of discrete-time reduced order models with
%          with Nq states.
%  *Q = Nx-by-Nq matrix of modes used to construct the reduced order model.
%  *Efit = Ngrid-by-1 vector of relative error for model (least-squares)
%       fits at each parameter grid point.

%% Set defaults
if nargin==5
    Nqmax = 10;
    Alg = 1;
elseif nargin==6
    Alg = 1;
end

%% Dimensions
Nu = size(U,1);
Ny = size(Y,1);
Ngrid = size(X,3);

%% Construct reduced order models
if Alg==1
    % Single-step method
    
    % Stack all state snapshots
    X0all = [];
    for i=1:Ngrid
        X0all = [X0all X(:,1:end-1,i)];
    end
    
    % POD modes using all state snapshots
    [UU,~,~]=svd(X0all);
    
    % Determine # of modes required for Efit<Etol at all grid points
    for Nq=1:Nqmax
        % Select modes for subspace projection
        Q = UU(:,1:Nq);
        
        % Compute fit errors at each grid point
        Efitrel = zeros(Ngrid,1);
        Gred = ss( zeros(Ny,Nu,Ngrid) );
        for i=1:Ngrid
            % Grab i^th data set
            Xi = X(:,:,i);
            Ui = U(:,:,i);
            Yi = Y(:,:,i);
            
            % Compute fit
            [Gred(:,:,i),~,~,Efitrel(i)]=IODMD(Xi,Ui,Yi,dt,Q);
        end
        
        % Quit if error tolerance is satisfied
        if all(Efitrel<Etol)
            break
        end
    end
else
    % Iterative Method
    for i=1:Ngrid
        % Grab i^th data set
        Xi = X(:,:,i);
        Ui = U(:,:,i);
        Yi = Y(:,:,i);
        
        % Add new basis vectors for Efit<Etol at the i^th point
        if i==1
            [~,Qvec,~,Efitrel]=IODMD(Xi,Ui,Yi,dt,1:Nqmax);
            idx = find(Efitrel<=Etol);
            Q = Qvec(:,1:idx(1));
        else
            Xnew = Xi - Q*Q'*Xi;
            [UU,~,~] = svd(Xnew);
            for j = 0:Nqmax
                Qnew = [Q UU(:,1:j)];
                [~,~,~,Efitrel]=IODMD(Xi,Ui,Yi,dt,Qnew);
                if Efitrel<Etol
                    Q = Qnew;
                    break
                end
            end
        end
    end
    
    % Compute reduced-order model using selected basis vectors
    Efitrel = zeros(Ngrid,1);
    Gred = ss( zeros(Ny,Nu,Ngrid) );
    for i=1:Ngrid
        [Gred(:,:,i),~,~,Efitrel(i)] = IODMD(X(:,:,i),U(:,:,i),Y(:,:,i),dt,Q);
    end    
end

